--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
LibFilter-1.0 -- Generic Filter Library

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Minimal library to support generic filters. The intent of the design is
that this library itself is not needed to implement or even use a filter.

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local MAJOR, MINOR = "LibFilter-1.0", 200+tonumber(("$Revision: 6 $"):match("(%d+)"))
local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

-- Cached functions
local _G      = getfenv(0)
local assert  = _G.assert
local type    = _G.type
local strfind = _G.strfind
local strsub  = _G.strsub
local strjoin = _G.strjoin
local next    = _G.next

local LibStub = _G.LibStub

-- Create or upgrade records
lib.FilterType = lib.FilterType or {}
lib.DecodeMeta = lib.DecodeMeta or {__mode='k'}
lib.Decode     = lib.Decode     or setmetatable({}, lib.DecodeMeta)

-- Local copies of members
local FilterType = lib.FilterType
local DecodeMeta = lib.DecodeMeta
local Decode = lib.Decode

-- Filter Types:
--  Blizard Types:
FilterType.ACHIEVEMENT	= "achievement"	-- * not sure of the format of this *
FilterType.ENCHANT	= "enchant"	-- enchant:ENCHANTID
FilterType.ITEM		= "item"	-- item:ITEMID:ENCHANTID:JEWEL1:JEWEL2:JEWEL3:JEWEL4:SUFFIX:UNIQUE
FilterType.QUEST	= "quest"	-- quest:QUESTID:QUESTLEVEL
FilterType.SPELL	= "spell"	-- spell:SPELLID
FilterType.TALENT	= "talent"	-- * not sure of the format of this *
--  LibSharedMedia Types
FilterType.BACKGROUND	= "Background"	-- Background:KEY
FilterType.TYPEFACE	= "Font"	-- Font:KEY
FilterType.FONT		= "Font"	-- Font:KEY
FilterType.BORDER	= "Border"	-- Border:KEY
FilterType.STATUSBAR	= "StatusBar"	-- StatusBar:KEY
FilterType.SOUND	= "Sound"	-- Sound:KEY
--  Other Types: (Feel free to create new ones)
FilterType.ADDON	= "Addon"	-- Addon:NAME
FilterType.PLAYER	= "Player"	-- Player:NAME:REALM
FilterType.SLOT		= "Slot"	-- Slot:BAGID:SLOTID


-- LibFilter:NewFilter(filtertype, key, version)
--   filtertype (string) - Type of filter
--   key (string) - Common name of the filter (should not be localized)
--   version (string or number) - Version of the filter
--
-- Convience function. Same as calling LibStub:NewLibrary("LibFilter-"..filtertype."-"..key,version)
-- directly, then setting the filtertype and key parameters.
function lib:NewFilter(filtertype, key, version)
	assert(type(filtertype) == "string", "Bad argument #2 to 'NewFilter' (string expected)")
	assert(type(key) == "string", "Bad argument #3 to 'NewFilter' (string expected)")
	-- No need to check version, lib filter will do that for use

	local filter = LibStub:NewLibrary("LibFilter-"..filtertype.."-"..key,version)
	if filter then
		filter.filtertype = filtertype
		filter.key = key
	end
	return filter
end

-- LibFilter:GetFilter(filtertype, key, [silent])
--   filtertype (string) - Type of filter
--   key (string) - Common name of the filter (should not be localized)
--   silent (boolean) - if true, filter is optional, slightly return nil if its not found
--
-- Convience function. You can call: LibStub("LibFilter-"..filtertype.."-"..key,silent) directly
function lib:GetFilter(filtertype, key, silent)
	assert(type(filtertype) == "string", "Bad argument #2 to 'NewFilter' (string expected)")
	assert(type(key) == "string", "Bad argument #3 to 'NewFilter' (string expected)")

	return LibStub:GetLibrary("LibFilter-"..filtertype.."-"..key,silent)
end

-- LibFilter:IterateFilters(filtertype)
--   filtertype (string) - Type of filter
--
-- Returns an iterator for the currently registered filters.
-- NOTE: This is not optimized... avoid calling it too often.
local function gettypednext(type)
	local matchtype = type
	local prefix = "LibFilter-"..type.."-"
	local function typednext(table, field)
		if field then field = prefix..field end
		local res
		for field, value in next, table, field do
			local first, last = strfind(field,prefix,1,1)
			if first and first == 1 then
				return strsub(field,last+1),value
			end
		end
		return nil
	end
	return typednext
end
function lib:IterateFilters(type)
	local _,libs = LibStub:IterateLibraries()
	return gettypednext(type),libs,nil
end

setmetatable(lib, { __call = lib.GetFilter })

-- LibFilter:Decode(linkorstring)
--   linkorstring (string ) - *link or *string to decode into a table.
--
-- Returns a decoded string as a table. The first index is the type of string
-- the rest are the values.
local nulltbl = {}
function DecodeMeta.__index(t,k)
	local value
	if k then
		local found, _, matched = strfind(k, "^|c%x+|H(.+)|h%[.*%]")
		if found then
			value = t[matched]
		else
			value = { full = k, strsplit(":", k) }
		end
	else
		value = nulltbl
	end
	rawset(t,k,value)
	return value
end

function DecodeMeta.__call(t,k,z) return t[z or k] end

-- LibFilter:SlotCode(bag,slot)
-- Not optimized
function lib:SlotCode(bag,slot)
	return strjoin(":",FilterType.SLOT,bag,slot)
end

-- What a generic filter should provide

-- Values:
--   filter.filtertype  - (required) filter of the filter
--   filter.key         - (required) key used for the filter (should not be localized)
--   filter.author      - (optional) name of the author of the filter
--   filter.credits     - (optional) additional credits
--   filter.name        - (optional) localized name (if not set most addons will use filter.key)
--   filter.description - (optional) longer localized description of filter

-- Methods:

-- filter:NewSettings([settings])
--   settings (table or nil) - Optional settings restored from saved variables.
--
-- Returns a new table with either the default settings or a copy of the settings passed in.
-- The contents of the settings table should be treated as opaque by the caller. If settings
-- have been freed or restored from saved variables they should be recreated using this function.

-- filter:FreeSettings(settings)
--   settings (table or nil) - Settings to be released
--
-- Allows the filter to clean up any record keeping relating to these settings. The contents of the
-- table should be unchanged and can be restored by calling filter:NewSettings on them. This should
-- be safe to call on settings that the filter does not know about.

-- filter:DescribeSettings
--   settings (table or nil) - Settings to be described or nil
--
-- Returns a string described the settings.

-- filter:SettingsOptions(settings,[func])
-- filter:SettingsOptions(settings,[table,method])
--   settings (table) - Settings to be adjust
--   func (function) - Function to be called with when settings changed
--   table (table) - Table to call method on
--   method (string) - Method to call
--
-- Returns an option table that can be used by AceOptions, Waterfall, Dewdrop etc to adjust the settings.
-- If a function or table and method are provided they are called when any of the settings are changed,
-- otherwise FilterUpdate should be fired. The settings table is passed to the func or method.
-- This method is optional and may not exist if the filter needs no options or may return nil.

-- filter:TestIfMatch(settings,linkorstring)
--   settings (table or nil) - Settings to be used for test. nil => default
--   linkorstring (string) - String to match
--
-- Returns non-nil if filter matches with settings otherwise nil

-- filter:TestIfMatchSplit(settings,....)
--   settings (table or nil) - Settings to be used for test. nil => default
--   ... - string split into pieces, e.g. filter:TestIfMatch(settings,"Slot",bag,slot)
--
-- This function is optional but an optimization if present.

-- Callback: FilterUpdate -- triggered if filter was updated and needs to be rescanned.

-- filter.RegisterCallback(target,"FilterUpdate",method,...)
-- filter.RegisterCallback(target,"FilterUpdate",func,...)
--
-- Calls target[method](...) or func(...) when the filter nedes to be rescanned.
-- This function is optional and if not present the filter is static.

-- filter.UnregisterCallback(target,"FilterUpdate")
-- filter.UnregisterAllCallbacks(target)
--
-- Removes callbacks for target. Again this function is optional.
