--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
Baggins_LibFilter

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Adds LibFilter support to Baggins

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

-- Need to decide which SVN server revision number I will use and fix this
-- probably will got with svn.wowace.com's version for the long run
local MAJOR, MINOR = "Baggins_LibFilter-1.0", tonumber(("$Revision: 6 $"):match("(%d+)"))
local lib = LibStub:NewLibrary(MAJOR,MINOR)
if not lib then return end

local LibFilter = LibStub("LibFilter-1.0")
local dewdrop

lib.known_rule_types = lib.known_rule_types or {}
lib.known_filters = lib.known_filters or {}
lib.known_settings = lib.known_settings or {}
lib.events = lib.events or {}

local known_rule_types = lib.known_rule_types
local known_filters = lib.known_filters
local known_settings = lib.known_settings
local events = lib.events
local do_upgrade = nil

local ITEM_TYPE = LibFilter.FilterType.ITEM
local SLOT_TYPE = LibFilter.FilterType.SLOT

local function Print(msg)
	local info = ChatTypeInfo["SYSTEM"]
	DEFAULT_CHAT_FRAME:AddMessage(msg, info.r, info.g, info.b, info.id)
end

local function FixSettings(filter,rule)
	local settings = rule.settings
	if settings then
		rule.no_settings = nil
		if not known_settings[settings] then
			settings = filter:NewSettings(settings)
			rule.settings = settings
			if settings then
				known_settings[settings] = true
			else
				rule.no_settings = true
			end
		end
	elseif not rule.no_settings then
		-- This effectively upgrades old rules provided the filter supports it
		settings = filter:NewSettings(rule)
		if settings then
			rule.settings = settings
			rule.no_settings = nil
			known_settings[settings] = true
		else
			rule.settings = nil
			rule.no_settings = true
		end
	end
	return settings
end

			
local function SlotMatchFast(bag,slot,rule)
	local filtername = rule.type
	local filter = known_filters[filtername]
	if not filter then return end

	return filter:TestIfMatchSplit(FixSettings(filter,rule),SLOT_TYPE,bag,slot)
end

local function SlotMatchSlow(bag,slot,rule)
	local filtername = rule.type
	local filter = known_filters[filtername]
	if not filter then return end

	return filter:TestIfMatch(FixSettings(filter,rule),strjoin(":",SLOT_TYPE,bag,slot))
end

local function ItemMatch(bag,slot,rule)
	local filtername = rule.type
	local filter = known_filters[filtername]
	if not filter then return end

	local link = GetContainerItemLink(bag, slot)
	return filter:TestIfMatch(FixSettings(filter,rule),link)
end

local function GetName(rule)
	local filtername = rule.type
	local filter = known_filters[filtername]
	if not filter then return filtername end

	local name
	if filter.DescribeSettings then
		name = filter:DescribeSettings(FixSettings(filter,rule))
	end

	return name or filter.name or filtername
end

local function CleanRule(filter, rule)
	local settings = rule.settings
	if (settings) then
		known_settings[settings] = nil
		filter:FreeSettings(settings)	
	end
	settings = filter:NewSettings()
	if settings then
		known_settings[settings] = 1
		rule.no_settings = nil
	else
		rule.no_settings = true
	end
	rule.settings = settings
end

local function RuleChanged()
	Baggins:OnRuleChanged()
	Baggins:ForceFullUpdate() -- Guess I need to call this also
end

function lib:update()
	Baggins:ForceFullUpdate()
end

local function DewDropOptions(rule, level, value)
	local filtername = rule.type
	local filter = known_filters[filtername]
	if not (filter and filter.SettingsOptions) then return end
	dewdrop = dewdrop or LibStub("Dewdrop-2.0")
	if not dewdrop then return end

	local opttable = filter:SettingsOptions(FixSettings(filter,rule),RuleChanged)
	if opttable then dewdrop:FeedAceOptionsTable(opttable) end
end

local function AddFilterRuleType(filtername,filter)
	if known_filters[filtername] and not do_upgrade then return end
	if was_known and not do_upgrade then return end
	known_filters[filtername] = filter
	local rule_type = known_rule_types[filtername] or {}
	rule_type.DisplayName = filter.name or filtername
	rule_type.Description = filter.description or filter.name or filtername
	rule_type.GetName = GetName
	if filter.filtertype == ITEM_TYPE then
		rule_type.Matches = ItemMatch
	else
		rule_type.Matches = filter.TestIfMatchSplit and SlotMatchFast or SlotMatchSlow
	end
	rule_type.CleanRule = function(rule) CleanRule(filter,rule) end
	rule_type.DewDropOptions = DewDropOptions
	-- Check to see if Baggins already has a known filter for this type
	local old_rule_type = Baggins:GetRuleDefinition(rulename)
	if not old_rule_type then
		-- Okay add the rule
		Print(("Baggins_LibFilter: Adapted |cFFFFFFFF%s|r"):format(filtername))
		Baggins:AddCustomRule(filtername,rule_type)
		if filter.RegisterCallback then
			filter.RegisterCallback(lib,"FilterUpdate","update")
		end
	elseif old_rule_type == rule_type then
		-- Any upgrade logic here if needed
		Print(("Baggins_LibFilter: Upgraded |cFFFFFFFF%s|r"):format(filtername))
	else
		-- Someone else owns this rule
		Print(("Baggins_LibFilter: |cFFFF0000Conflict|r |cFFFFFFFF%s|r - Not registered"):format(filtername))
	end
end

local function UpdateFilters()
	if Baggins and Baggins.AddCustomRule then
		for filtername,filter in LibFilter:IterateFilters(SLOT_TYPE) do
			AddFilterRuleType(filtername,filter)
		end
		for filtername,filter in LibFilter:IterateFilters(ITEM_TYPE) do
			AddFilterRuleType(filtername,filter)
		end
	end
end

do_upgrade = true
UpdateFilters()
do_upgrade = nil

lib.myFrame = lib.myFrame or CreateFrame("Frame","Baggins_LibFilter_Frame")
local myFrame = lib.myFrame
myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)

function events.ADDON_LOADED()
	UpdateFilters()
end

myFrame:RegisterEvent("ADDON_LOADED");

function events.UPDATE()
	UpdateFilters()
	myFrame:SetScript("OnUpdate",nil)
end

function events.PLAYER_LOGIN()
	myFrame:SetScript("OnUpdate",function (self, ...) events.UPDATE(...) end)
end
myFrame:RegisterEvent("PLAYER_LOGIN");
