--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
LibFilter_ClosetGnome

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Thanks to Rabbit for writing ClosetGnome

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local MAJOR, MINOR = "LibFilter-item-ClosetGnome", tonumber(("$Revision: 6 $"):match("(%d+)"))
local filter = LibStub:NewLibrary(MAJOR, MINOR)
if not filter then return end

filter.filtertype = "item"
filter.key = "ClosetGnome"

filter.callbacks = filter.callbacks or LibStub("CallbackHandler-1.0"):New(filter)
local callbacks = filter.callbacks

local ipairs = ipairs
local next = next
local pairs = pairs
local strmatch = string.match
local strjoin = strjoin
local tinsert = table.insert
local tsort = table.sort

local items_table = {}
local sets_table = {}

-- No need for a full blown locale library here
local L
do
	local LL = {
		enUS = {
			["ClosetGnome Set:%s"] = true,
			["Unknown"] = true,
			["ClosetGnome Sets"] = true,
			["Any"] = true,
			["None"] = true,
			["ClosetGnome"] = true,
			["Filter by ClosetGnome set"] = true,
			["Fuzzy match"] = true,
			["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = true,
		},
		koKR = {
			["ClosetGnome Set:%s"] = "ClosetGnome 세트명:%s",
			["Unknown"] = "가격미확인", -- check
			["ClosetGnome Sets"] = "ClosetGnome 세트",
			["Set Name"] = "세트명",
			["Any"] = "모두",
			["None"] = "없음", -- check
			--["ClosetGnome"] = true,
			["Filter by ClosetGnome set"] = "ClosetGnome 세트에 따라 필터링",
			["Fuzzy match"] = "미세분화 분류",
			["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = "마법부여나 키트 효과등을 무시하고 아이템 ID에 따라 분류합니다",
		},
	}

	local gameLocale = GetLocale()
	local enL = LL["enUS"]
	L = LL[gameLocale] or LL["enUS"]

	for k in pairs(enL) do
		if not LL[k] then LL[k] = k end
	end
	for k,v in pairs(L) do
		if v == true then L[k] = k end
	end
end


-- Prototypes
local linkToIds -- (link)
local idToFuzzy -- (id)
local CacheSetItem -- (set, item, table1, table2, ...)
local CacheSets -- ()

local UpgradeSettings -- (settings)
local ExpandSettings -- (settings)
local NormalizeSettings -- (settings)

local function linkToIds(link)
	if type(link) == "string" then
		local first,fuzzy,last = link:match("item:((%-?%d+):%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:)(%-?%d+)")
		if first and last then
			last = last % 65536
			return first..last,fuzzy
		end
	end
	return nil,nil
end

local function idToFuzzy(id)
	if type (id) == "string" then
		return id:match("(%-?%d+)")
	else
		return id
	end
end

function CacheSetItem(set, item, ...)
	if item then
		local fuzzy = idToFuzzy(item)

		for i = 1,select("#",...) do
			local current_table = select(i,...)

			if current_table then
				if not current_table[item] then current_table[item] = {} end
				current_table[item][set] = true

				if fuzzy ~= item then
					if not current_table[fuzzy] then current_table[fuzzy] = {} end
					current_table[fuzzy][set] = true
				end
			end
		end
	end
end

function CacheSets()
	local i,set

	local sets = ClosetGnome.db.char.set

	items_table = {}
	sets_table = {}
	for setname, set in pairs(sets) do
		sets_table[setname] = true
		for i=0,19 do
			CacheSetItem(setname,linkToIds(set[i]),items_table)
		end
	end
end

function UpgradeSettings(settings)
	local sets = settings.sets
	if not sets then
		local set = settings.set
		if not set or set == "Any" then
			sets = { Any = true }
		elseif set == "None" then
			sets = { }
		else
			sets = { [set] = true }
		end
		settings.sets = sets
	end
	return sets
end

function ExpandSettings(settings)
	local sets = settings.sets or UpgradeSettings(settings)

	if sets.Any then
		for set in pairs(sets_table) do
			sets[set] = true
		end
	end
	return sets
end

function NormalizeSettings(settings)
	local sets = settings.sets or UpgradeSettings(settings)

	local nextset = next(sets)

	-- No sets selected
	if not nextset then
		settings.set = "None"
		return sets
	end

	-- Just one set selected
	if not next(sets,nextset) then
		settings.set = nextset
		return sets
	end

	-- Okay check if all sets are selected
	sets.Any = true
	for set in pairs(sets_table) do
		if not sets[set] then sets.Any = nil end
	end

	if sets.Any then
		sets = { Any = true }
		settings.sets = sets
		settings.set = "Any"
		return sets
	end

	settings.set = "????"
	return sets
end

local events = {}
local myFrame = CreateFrame("Frame","LibFilter_ClosetGnome_Frame")
myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)

local function update()
	callbacks:Fire("FilterUpdate")
end

function filter:CacheSetsAndUpdate(...)
	CacheSets()
	-- Should really only fire cache changed
	update()
end

local hooked
local function HookClosetGnome()
	if hooked then return end
	if not ClosetGnome then return end
	if not ClosetGnome.db then return end
	if not ClosetGnome.db.char then return end
	hooked = true
	ClosetGnome.RegisterCallback(filter,"AddSet", "CacheSetsAndUpdate")
	ClosetGnome.RegisterCallback(filter,"UpdateSet", "CacheSetsAndUpdate")
	ClosetGnome.RegisterCallback(filter,"DeleteSet", "CacheSetsAndUpdate")
	CacheSets()
	myFrame:UnregisterEvent("ADDON_LOADED")
	myFrame:SetScript("OnUpdate",nil)
end

events.ADDON_LOADED = HookClosetGnome
events.UPDATE = HookClosetGnome
function events.PLAYER_LOGIN()
	myFrame:SetScript("OnUpdate",function (self, ...) events.UPDATE(...) end)
end

myFrame:RegisterEvent("ADDON_LOADED")
myFrame:RegisterEvent("PLAYER_LOGIN")

filter.author = "Nayala"
filter.credits = "ClosetGnome was written by Rabbit"
filter.name = L["ClosetGnome"]
filter.description = L["Filter by ClosetGnome set"]

function filter:NewSettings(settings)
	local res = {
		Any = settings and settings.Any or false,
		set = settings and settings.set,
		fuzzy = settings and settings.fuzzy or false,
	}
	if settings and settings.sets then
		local sets = {}
		for k,v in pairs(settings.sets) do
			sets[k] = v
		end
		res.sets = sets
	else
		UpgradeSettings(res)
	end
	return res
end

function filter:FreeSettings(settings)
end

function filter:DescribeSettings(settings)
	local msg

	local sets = settings.sets or UpgradeSettings(settings)
	local setlist
	if not next(sets) then
		setlist = L["None"]
	elseif sets.Any then
		setlist = L["Any"]
	else
		local setnames = {}
		for set in pairs(sets) do
			tinsert(setnames,set)
		end
		tsort(setnames)
		setlist = strjoin(", ",unpack(setnames))
	end

	msg = L["ClosetGnome Set:%s"]:format(setlist)
	if settings.fuzzy then
		msg = msg.." "..L["Fuzzy match"]
	end
	return msg
end

local function toggleSettingsSet(settings, set)
	if set == "Any" then
		settings.set = "Any"
		settings.sets = { Any = true }
	elseif set == "None" then
		settings.set = "None"
		settings.sets = { }
	else
		local sets = ExpandSettings(settings)

		-- Okay update the given set
		sets[set] = not sets[set] and true or nil

		-- Okay normalize
		sets.Any = nil -- Since we are expanded, can clear Any
		NormalizeSettings(settings)
	end
end

local function toggleSettingsFuzzy(settings)
	settings.fuzzy = not settings.fuzzy
end


function filter:SettingsOptions(settings,table,method)
	local callback = update

	if table then
		if type(table) == "string" then
			callback = function() table[func](table,settings) end
		else
			callback = function() table(func,settings) end
		end
	end

	local args = {
		{
			order = 1,
			type = "header",
			name = L["ClosetGnome Sets"],
		},
		{
			order = -1,
			type = "toggle",
			name = L["Fuzzy match"],
			desc = L["Match by the item number, ignoring enchantments, variations, armor kits, etc."],
			get = function() return settings.fuzzy end,
			set = function(v)
							toggleSettingsFuzzy(settings)
							callback()
						end,
		},
		{
			order = 3,
			type = "toggle",
			name = L["Any"],
			desc = L["Any"],
			get = function() return settings.sets.Any end,
			set = function(v)
							toggleSettingsSet(settings,"Any")
							callback()
						end,
		},
		{
			order = 4,
			type = "toggle",
			name = L["None"],
			desc = L["None"],
			get = function() return not next(settings.sets) end,
			set = function(v)
							toggleSettingsSet(settings,"None")
							callback()
						end,
		},
	}
	local setnames = {}
	for set in pairs(sets_table) do
		tinsert(setnames,set)
	end
	tsort(setnames)
	local order = 4
	for _,set in ipairs(setnames) do
		order = order + 1
		args[order] = {
			type = "toggle",
			order = order,
			name = set,
			desc = set,
			get = function() return settings.sets[set] or settings.sets.Any end,
			set = function(v)
							toggleSettingsSet(settings,set)
							callback()
						end,
		}
	end
	return { type = "group", desc = "LibFilter_ClosetGnome options", args = args }
end

function filter:TestIfMatch(settings,link)
	local id,idfuzzy = linkToIds(link)
	if not id then return end

	local sets = settings.sets or UpgradeSettings(settings)
	local any = sets.Any
	local none = not next(sets)

	if settings.fuzzy then
		id = idfuzzy
		if not id then return end
	end

	local res = items_table
	if res then
		if id and res[id] then
			res = res[id]
		else
			res = nil
		end
	end
	if none then
		return not res
	elseif any or not res then
		return res
	else
		for set in pairs(sets) do
			if res[set] then return res end
		end
	end
	return nil
end

