--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
LibFilter_ItemRack

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Thanks to Gello for writing ItemRack

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local MAJOR, MINOR = "LibFilter-item-ItemRack", tonumber(("$Revision: 6 $"):match("(%d+)"))
local filter = LibStub:NewLibrary(MAJOR, MINOR)
if not filter then return end

filter.filtertype = "item"
filter.key = "ItemRack"

filter.callbacks = filter.callbacks or LibStub("CallbackHandler-1.0"):New(filter)
local callbacks = filter.callbacks

local ipairs = ipairs
local next = next
local pairs = pairs
local strmatch = string.match
local strjoin = strjoin
local tinsert = table.insert
local tsort = table.sort

local items_table = {}
local sets_table = {}

-- No need for a full blown locale library here
local L
do
	local LL = {
		enUS = {
			["ItemRack Set:%s"] = true,
			["Unknown"] = true,
			["ItemRack Sets"] = true,
			["Set Name"] = true,
			["Any"] = true,
			["None"] = true,
			["ItemRack"] = true,
			["Filter by ItemRack set"] = true,
			["Fuzzy match"] = true,
			["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = true,
		},
		koKR = {
			["ItemRack Set:%s"] = "ItemRack 세트명:%s",
			["Unknown"] = "가격미확인", -- check
			["ItemRack Sets"] = "ItemRack 세트",
			["Set Name"] = "세트명",
			["Any"] = "모두",
			["None"] = "없음", -- check
			--["ItemRack"] = true,
			["Filter by ItemRack set"] = "ItemRack 세트에 따라 필터링",
			["Fuzzy match"] = "미세분화 분류",
			["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = "마법부여나 키트 효과등을 무시하고 아이템 ID에 따라 분류합니다",
		},
	}

	local gameLocale = GetLocale()
	local enL = LL["enUS"]
	L = LL[gameLocale] or LL["enUS"]

	for k in pairs(enL) do
		if not LL[k] then LL[k] = k end
	end
	for k,v in pairs(L) do
		if v == true then L[k] = k end
	end
end


-- Prototypes
local linkToIds -- (link)
local idToFuzzy -- (id)
local CacheSetItem -- (set, item, table1, table2, ...)
local CacheSets -- ()

local UpgradeSettings -- (settings)
local ExpandSettings -- (settings)
local NormalizeSettings -- (settings)

function linkToIds(link)
	if type (link) == "string" then
		return strmatch(link,"(((%-?%d+):%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+):%-?%d+)")
	end
	return nil,nil,nil
end

function idToFuzzy(id)
	if type (id) == "string" then
		return strmatch(id,"(%-?%d+)")
	else
		return id
	end
end

function CacheSetItem(set, item, ...)
	if item then
		local fuzzy = idToFuzzy(item)

		for i = 1,select("#",...) do
			local current_table = select(i,...)

			if current_table then
				if not current_table[item] then current_table[item] = {} end
				current_table[item][set] = true

				if fuzzy ~= item then
					if not current_table[fuzzy] then current_table[fuzzy] = {} end
					current_table[fuzzy][set] = true
				end
			end
		end
	end
end

function CacheSets()
	-- Normal sets
	local sets = ItemRackUser.Sets

	items_table = {}
	sets_table = {}
	for setname, set in pairs(sets) do
		if not strmatch(setname,"^~") then
			sets_table[setname] = true
			for slot=0,19 do
				CacheSetItem(setname,linkToIds(set.equip[slot]),items_table)
			end
		end
	end

	-- Queues
	local queues = ItemRackUser.Queues

	local setname = "Queue"
	sets_table[setname] = true
	for slot, queue in pairs(queues) do
		for order,setitem in ipairs(queue) do
			if setitem == 0 then break end
			CacheSetItem(setname,linkToIds(setitem),items_table)
		end
	end
end

function UpgradeSettings(settings)
	local sets = settings.sets
	if not sets then
		local set = settings.set
		if not set or set == "Any" then
			sets = { Any = true }
		elseif set == "None" then
			sets = { }
		else
			sets = { [set] = true }
		end
		settings.sets = sets
	end
	return sets
end

function ExpandSettings(settings)
	local sets = settings.sets or UpgradeSettings(settings)

	if sets.Any then
		for set in pairs(sets_table) do
			sets[set] = true
		end
	end
	return sets
end

function NormalizeSettings(settings)
	local sets = settings.sets or UpgradeSettings(settings)

	local nextset = next(sets)

	-- No sets selected
	if not nextset then
		settings.set = "None"
		return sets
	end

	-- Just one set selected
	if not next(sets,nextset) then
		settings.set = nextset
		return sets
	end

	-- Okay check if all sets are selected
	sets.Any = true
	for set in pairs(sets_table) do
		if not sets[set] then sets.Any = nil end
	end

	if sets.Any then
		sets = { Any = true }
		settings.sets = sets
		settings.set = "Any"
		return sets
	end

	settings.set = "????"
	return sets
end


local events = {}
local myFrame = CreateFrame("Frame","LibFilter_ItemRack_Frame")
myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)

local function update()
	callbacks:Fire("FilterUpdate")
end

function CacheSetsAndUpdate()
	CacheSets()
	-- Should really only fire cache changed
	update()
end

local orig = {}
local hooks = {}

function hooks.SaveSet(...)
	orig.SaveSet(...)
	CacheSetsAndUpdate()
end

function hooks.DeleteSet(...)
	orig.DeleteSet(...)
	CacheSetsAndUpdate()
end

local options_hooked
local function HookOptions()
	if options_hooked then return end
	if not ItemRackOpt then return end
	options_hooked = true
	orig.SaveSet = ItemRackOpt.SaveSet
	orig.DeleteSet = ItemRackOpt.DeleteSet
	ItemRackOpt.SaveSet = hooks.SaveSet
	ItemRackOpt.DeleteSet = hooks.DeleteSet
end


function hooks.InitCore(...)
	orig.InitCore(...)
	CacheSetsAndUpdate()
end

function hooks.ToggleOptions(...)
	orig.ToggleOptions(...)
	HookOptions()
end

local hooked
local function HookItemRack()
	if hooked then return end
	if not ItemRack then return end
	if ItemRack_Version and ItemRack_Version < 2 then
		assert("LibFilter_ItemRack; Unsupported version of ItemRack")
	end
	hooked = true
	orig.InitCore = ItemRack.InitCore
	ItemRack.InitCore = hooks.InitCore
	if ItemRackOpt then
		HookOptions()
	else
		orig.ToggleOptions = ItemRack.ToggleOptions
		ItemRack.ToggleOptions = hooks.ToggleOptions
	end
	myFrame:UnregisterEvent("ADDON_LOADED")
end

events.ADDON_LOADED = HookItemRack
events.PLAYER_LOGIN = HookItemRack
myFrame:RegisterEvent("ADDON_LOADED")
myFrame:RegisterEvent("PLAYER_LOGIN")

filter.author = "Nayala"
filter.credits = "ItemRack was written by Gello"
filter.name = L["ItemRack"]
filter.description = L["Filter by ItemRack set"]

function filter:NewSettings(settings)
	local res = {
		Any = settings and settings.Any or false,
		set = settings and settings.set,
		fuzzy = settings and settings.fuzzy or false,
	}
	if settings and settings.sets then
		local sets = {}
		for k,v in pairs(settings.sets) do
			sets[k] = v
		end
		res.sets = sets
	else
		UpgradeSettings(res)
	end
	return res
end

function filter:FreeSettings(settings)
end

function filter:DescribeSettings(settings)
	local msg

	local sets = settings.sets or UpgradeSettings(settings)
	local setlist
	if not next(sets) then
		setlist = L["None"]
	elseif sets.Any then
		setlist = L["Any"]
	else
		local setnames = {}
		for set in pairs(sets) do
			tinsert(setnames,set)
		end
		tsort(setnames)
		setlist = strjoin(", ",unpack(setnames))
	end

	msg = L["ItemRack Set:%s"]:format(setlist)
	if settings.fuzzy then
		msg = msg.." "..L["Fuzzy match"]
	end
	return msg
end

local function toggleSettingsSet(settings, set)
	if set == "Any" then
		settings.set = "Any"
		settings.sets = { Any = true }
	elseif set == "None" then
		settings.set = "None"
		settings.sets = { }
	else
		local sets = ExpandSettings(settings)

		-- Okay update the given set
		sets[set] = not sets[set] and true or nil

		-- Okay normalize
		sets.Any = nil -- Since we are expanded, can clear Any
		NormalizeSettings(settings)
	end
end

local function toggleSettingsFuzzy(settings)
	settings.fuzzy = not settings.fuzzy
end

function filter:SettingsOptions(settings,table,method)
	local callback = update

	if table then
		if type(table) == "string" then
			callback = function() table[func](table,settings) end
		else
			callback = function() table(func,settings) end
		end
	end

	local sets = settings.sets or UpgradeSettings(settings)

	local args = {
		{
			order = 1,
			type = "header",
			name = L["ItemRack Sets"],
		},
		{
			order = -1,
			type = "toggle",
			name = L["Fuzzy match"],
			desc = L["Match by the item number, ignoring enchantments, variations, armor kits, etc."],
			get = function() return settings.fuzzy end,
			set = function(v)
							toggleSettingsFuzzy(settings)
							callback()
						end,
		},
		{
			order = 3,
			type = "toggle",
			name = L["Any"],
			desc = L["Any"],
			get = function() return settings.sets.Any end,
			set = function(v)
							toggleSettingsSet(settings,"Any")
							callback()
						end,
		},
		{
			order = 4,
			type = "toggle",
			name = L["None"],
			desc = L["None"],
			get = function() return not next(settings.sets) end,
			set = function(v)
							toggleSettingsSet(settings,"None")
							callback()
						end,
		},
	}
	local setnames = {}
	for set in pairs(sets_table) do
		tinsert(setnames,set)
	end
	tsort(setnames)
	local order = 4
	for _,set in ipairs(setnames) do
		order = order + 1
		args[order] = {
			type = "toggle",
			order = order,
			name = set,
			desc = set,
			get = function() return settings.sets[set] or settings.sets.Any end,
			set = function(v)
							toggleSettingsSet(settings,set)
							callback()
						end,
		}
	end
	return { type = "group", desc = "LibFilter_ItemRack options", args = args }
end

function filter:TestIfMatch(settings,link)
	local id,idbug,idfuzzy = linkToIds(link)
	if not id then return end
	if not idbug then idbug = id end

	local sets = settings.sets or UpgradeSettings(settings)
	local any = sets.Any
	local none = not next(sets)

	if settings.fuzzy then
		id = idfuzzy
		idbug = idfuzzy
		if not id then return end
	end

	local res = items_table
	if res then
		if id and res[id] then
			res = res[id]
		elseif idbug and res[idbug] then
			res = res[idbug]
		else
			res = nil
		end
	end
	if none then
		return not res
	elseif any or not res then
		return res
	else
		for set in pairs(sets) do
			if res[set] then return res end
		end
	end
	return nil
end
